
from qt import *
from qtcanvas import *

TILESIZE = 64

class CanvasView(QCanvasView):

    def __init__(self, *args):
        QCanvasView.__init__(self, *args)
        self.tileTimer = QTimer()
        self.closeImage()       # create other initial instance vars
        self.connect(self.tileTimer, SIGNAL("timeout()"), self.processOneTile)


    def closeImage(self):
        print "CanvasView.closeImage"
        self.setCanvas(QCanvas(self))
        print "CanvasView.closeImage resize", self.canvas()
        self.canvas().resize(0, 0)
        print "CanvasView.closeImage update"
        self.canvas().update()

        # shouldn't be necessary
        #for i in self.tiles.keys():
        #    del self.tiles[i]
        #for i in self.sprites.keys():
        #    del self.sprites[i]
        #for i in self.sequences.keys():
        #    del self.sequences[i]

        self.tiles = {}
        self.sprites = {}
        self.sequences = {}

        self.tilesx = 0
        self.tilesy = 0
        self.tileToProcess = 0
        self.tileTimer.stop()
        self.lastTable = []
        self.imagesize = QSize(0, 0)
        self.filename = None
        print "CanvasView.closeImage finished"


    def loadImage(self, filename):
        if self.imagesize.isValid():
            self.closeImage()

        self.filename = filename
        img = QImage(filename)
        self.imagesize = img.size()

        self.tilesx = img.width() / TILESIZE
        if img.width() % TILESIZE:
            self.tilesx += 1
        self.tilesy = img.height() / TILESIZE
        if img.height() % TILESIZE:
            self.tilesy += 1

        self.canvas().resize(img.width(), img.height())

        for y in range(self.tilesy):
            for x in range(self.tilesx):
                self.tiles[y * self.tilesx + x] = \
                    img.copy(TILESIZE * x, TILESIZE * y, TILESIZE, TILESIZE)
                sequence = []
                px = QPixmap()
                px.convertFromImage(self.tiles[y * self.tilesx + x])
                sequence.append(px)
                self.sequences[y * self.tilesx + x] = QCanvasPixmapArray(sequence)
                spr = QCanvasSprite(self.sequences[y * self.tilesx + x], self.canvas())
                spr.move(TILESIZE * x, TILESIZE * y)
                spr.show()
                self.sprites[y * self.tilesx + x] = spr
        self.canvas().update()
        #print "tiles:", self.tiles
        #print "sequences:", self.sequences
        #print "sprites:", self.sprites


    def saveImage(self):
        img = QImage(self.imagesize, 32)
        for y in range(self.tilesy):
            for x in range(self.tilesx):
                self.bitBlt(img, x * TILESIZE, y * TILESIZE, self.tiles[y * self.tilesx + x])
        img.save(self.filename, "PNG")


    def commit(self):
        for i in range(len(self.tiles)):
            for y in range(TILESIZE):
                for x in range(TILESIZE):
                    pixel = self.tiles[i].pixel(x, y)
                    self.tiles[i].setPixel(x,y, qRgb(
                            self.lastTable[qRed(pixel)],
                            self.lastTable[qGreen(pixel)],
                            self.lastTable[qBlue(pixel)]))


    def rollback(self):
        for i in range(len(self.tiles)):
            self.sequences[i].setImage(0, QCanvasPixmap(self.tiles[i]))
            self.sprites[i].setSequence(self.sequences[i])
            self.canvas().update()


    def setFileName(self, filename):
        self.filename = filename


    def applyCurve(self, curve):
        self.lastTable = curve
        self.tileToProcess = 0
        self.tileTimer.start(0)


    def processOneTile(self):
        timg = QImage(TILESIZE, TILESIZE, 32)
        for y in range(TILESIZE):
            for x in range(TILESIZE):
                pixel = self.tiles[self.tileToProcess].pixel(x,y)
                timg.setPixel(x,y, qRgb(
                    self.lastTable[qRed(pixel)],
                    self.lastTable[qGreen(pixel)],
                    self.lastTable[qBlue(pixel)]))

        self.sequences[self.tileToProcess].setImage(0, QCanvasPixmap(timg))
        self.sprites[self.tileToProcess].setSequence(self.sequences[self.tileToProcess])
        self.canvas().update()
        self.tileToProcess += 1
        if self.tileToProcess == len(self.tiles):
            self.tileTimer.stop()
