#!/usr/bin/env python

import sys
from qt import *
from mainwindowbase import MainWindowBase
from contrastdialog import ContrastDialog
from valuedialog import ValueDialog

class MainWindow(MainWindowBase):

    def __init__(self, parent = None, name = None, fl = 0):
        #MainWindowBase.__init__(self, parent, name, fl + Qt.WDestructiveClose)
        MainWindowBase.__init__(self, parent, name, fl)
        self.contrastDialog = None
        self.valueDialog = None
        self.statusBar().setSizeGripEnabled(False)
        self.connect(self.helpAboutQtAction, SIGNAL("activated()"),
                     qApp, SLOT("aboutQt()"))


    def openFile(self):
        filename = QFileDialog.getOpenFileName()
        if not filename.isNull():
            fi = QFileInfo(filename)

            self.canvas.loadImage(filename)
            self.imageContrastAction.setEnabled(True)
            self.imageValuesAction.setEnabled(True)
            self.fileSaveAction.setEnabled(True)
            self.fileSaveAsAction.setEnabled(True)
            self.fileCloseAction.setEnabled(True)
            self.setCaption(QString("Bildbearbeitung - %1").arg(fi.fileName()))


    def closeFile(self):
        self.canvas.closeImage()
        self.imageContrastAction.setEnabled(False)
        self.imageValuesAction.setEnabled(False)
        self.fileSaveAction.setEnabled(False)
        self.fileSaveAsAction.setEnabled(False)
        self.fileCloseAction.setEnabled(False)
        self.setCaption(QString("Bildbearbeitung - Keine Datei"))


    def saveFile(self):
        self.canvas.saveImage()


    def saveFileAs(self):
        filename = QFileDialog.getSaveFileName()
        if not filename.isNull():
            self.canvas.setFileName(filename)
            self.canvas.saveImage()
            fi = QFileInfo(filename)
            self.setCaption(QString("Bildbearbeitung - %1").arg(fi.fileName()))


    def contrastBrightness(self):
        if not self.contrastDialog:
            self.contrastDialog = ContrastDialog(self)
            self.connect(self.contrastDialog, PYSIGNAL("curveChanged"),
                         self.canvas.applyCurve)

        if self.contrastDialog.exec_loop() == QDialog.Accepted:
            self.canvas.commit()
        else: # if QDialog.Rejected
            self.canvas.rollback()


    def values(self):
        if not self.valueDialog:
            self.valueDialog = ValueDialog(self)
            self.connect(self.valueDialog, PYSIGNAL("curveChanged"),
                         self.canvas.applyCurve)

        if self.valueDialog.exec_loop() == QDialog.Accepted:
            self.canvas.commit()
        else: # of QDialog.Rejected:
            self.canvas.rollback()


    def close(self, alsodelete = False):
        print "MainWindow.close", alsodelete
        self.closeFile()
        return MainWindowBase.close(self, alsodelete)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    cv = MainWindow()
    app.setMainWidget(cv)
    cv.show()
    app.exec_loop()
